unit Functions;

interface

uses
{$IFDEF FPC}
  LResources,
{$ENDIF}
{$IFNDEF LINUX}
  Windows, Messages, Variants,
{$ENDIF}
  SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  DBCtrls, ExtCtrls, Db, Grids, DBGrids, StdCtrls, ToolWin, ComCtrls, Buttons,
{$IFNDEF FPC}
  MemDS,
{$ELSE}
  MemDataSet,
{$ENDIF}
  LiteDACVcl, DemoFrame, LiteDacDemoForm, DBAccess, LiteAccess;

type
  TFunctionsFrame = class(TDemoFrame)
    DataSource: TDataSource;
    LiteQuery: TLiteQuery;
    DBGrid: TDBGrid;
    Panel9: TPanel;
    DBNavigator: TDBNavigator;
    Splitter1: TSplitter;
    ToolBar: TPanel;
    Panel1: TPanel;
    btClose: TSpeedButton;
    btOpen: TSpeedButton;
    Memo: TMemo;
    fnPercent: TLiteUserFunction;
    fnABS: TLiteUserFunction;
    fnPercent_1: TLiteUserFunction;
    procedure btOpenClick(Sender: TObject);
    procedure btCloseClick(Sender: TObject);
    procedure DataSourceDataChange(Sender: TObject; Field: TField);
    procedure DataSourceStateChange(Sender: TObject);
    procedure LiteQueryAfterOpen(DataSet: TDataSet);
    procedure fnPercentExecute(Sender: TObject; Params: TDAParams; var ResultValue: Variant);
    procedure fnABSExecute(Sender: TObject; Params: TDAParams; var ResultValue: Variant);
    procedure fnPercent_1Execute(Sender: TObject; Params: TDAParams; var ResultValue: Variant);
  private
    procedure ShowState;
  public
    // Demo management
    procedure Initialize; override;
    procedure SetDebug(Value: boolean); override;
  end;

implementation

{$IFNDEF FPC}
{$IFDEF CLR}
{$R *.nfm}
{$ELSE}
{$R *.dfm}
{$ENDIF}
{$ENDIF}

{ User-defined function }

procedure TFunctionsFrame.fnPercentExecute(Sender: TObject; Params: TDAParams; var ResultValue: Variant);
var
  lComm, lSal: real;
begin
  lComm := Params[0].AsFloat;
  lSal := Params[1].AsFloat;
  if lSal = 0 then
    lSal := 1;
  ResultValue := lComm/lSal*100;
end;

{ Overloaded function with a different number of parameters }

procedure TFunctionsFrame.fnPercent_1Execute(Sender: TObject; Params: TDAParams; var ResultValue: Variant);
begin
  ResultValue := VarToStrDef(Params[0].Value, '0') + ' %';
end;

{ Overriding the built-in function.
  Built-in Abs(X) returns NULL if X is NULL.
  Overrided function will return 0 if X is NULL }

procedure TFunctionsFrame.fnABSExecute(Sender: TObject; Params: TDAParams; var ResultValue: Variant);
begin
  ResultValue := Abs(Params[0].AsInteger);
end;

{ TFunctionsFrame }

procedure TFunctionsFrame.ShowState;
var
  St: string;

  procedure AddSt(S:string);
  begin
    if St <> '' then
      St := St + ', ';
    St := St + S;
  end;

begin
  St := '';

  if LiteQuery.Active then
    AddSt('Active')
  else
    AddSt('Inactive');

  if LiteQuery.Executing then
    AddSt('Executing');

  if LiteQuery.Fetching then
    AddSt('Fetching');

  LiteDacForm.StatusBar.Panels[2].Text := St;
end;

procedure TFunctionsFrame.btOpenClick(Sender: TObject);
begin
  try
    LiteQuery.SQL.Text := Memo.Lines.Text;
    LiteQuery.Open;
  finally
    ShowState;
  end;
end;

procedure TFunctionsFrame.btCloseClick(Sender: TObject);
begin
  LiteQuery.Close;
  ShowState;
end;

procedure TFunctionsFrame.DataSourceStateChange(Sender: TObject);
begin
  LiteDacForm.StatusBar.Panels[1].Text := 'Record ' + IntToStr(LiteQuery.RecNo) + ' of ' + IntToStr(LiteQuery.RecordCount);
end;

procedure TFunctionsFrame.DataSourceDataChange(Sender: TObject; Field: TField);
begin
  DataSourceStateChange(nil);
end;

procedure TFunctionsFrame.LiteQueryAfterOpen(DataSet: TDataSet);
begin
  ShowState;
end;

procedure TFunctionsFrame.Initialize;
begin
  inherited;
  LiteQuery.Connection := Connection as TLiteConnection;

  Memo.Lines.Text := LiteQuery.SQL.Text;
end;

procedure TFunctionsFrame.SetDebug(Value: boolean);
begin
  LiteQuery.Debug := Value;
end;

{$IFDEF FPC}

initialization
  {$i Functions.lrs}
{$ENDIF}

end.

