unit Table;

interface

uses
{$IFDEF FPC}
  LResources,
{$ENDIF}
{$IFNDEF LINUX}
  Windows, Messages,
{$ENDIF}
  SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  DBCtrls, ExtCtrls, Db, Grids, DBGrids, StdCtrls, ToolWin,
  ComCtrls, Buttons, LiteDACVcl, DemoFrame, LiteDacDemoForm,
  CRTypes, CRFunctions,
  DBAccess, LiteAccess,
  {$IFNDEF FPC}MemDS{$ELSE}MemDataSet{$ENDIF};

type
  TTableFrame = class(TDemoFrame)
    DataSource: TDataSource;
    LiteTable: TLiteTable;
    DBGrid: TDBGrid;
    ToolBar: TPanel;
    Panel3: TPanel;
    btOpen: TSpeedButton;
    btClose: TSpeedButton;
    btPrepare: TSpeedButton;
    btUnPrepare: TSpeedButton;
    btExecute: TSpeedButton;
    DBNavigator: TDBNavigator;
    Panel7: TPanel;
    Label3: TLabel;
    edOrderFields: TEdit;
    Panel5: TPanel;
    Label1: TLabel;
    edTableName: TComboBox;
    Panel8: TPanel;
    Label4: TLabel;
    edFilterSQL: TEdit;
    procedure btOpenClick(Sender: TObject);
    procedure btCloseClick(Sender: TObject);
    procedure btPrepareClick(Sender: TObject);
    procedure btUnPrepareClick(Sender: TObject);
    procedure btExecuteClick(Sender: TObject);
    procedure edTableNameExit(Sender: TObject);
    procedure edTableNameDropDown(Sender: TObject);
  private
    { Private declarations }
    procedure ShowState;
    procedure AssignProperties;
  public
    // Demo management
    procedure Initialize; override;
    procedure SetDebug(Value: boolean); override;
  end;

implementation

{$IFNDEF FPC}
{$IFDEF CLR}
{$R *.nfm}
{$ELSE}
{$R *.dfm}
{$ENDIF}
{$ENDIF}

procedure TTableFrame.AssignProperties;
begin
  try
    if Trim(LiteTable.TableName) <> Trim(edTableName.Text) then
      LiteTable.TableName := edTableName.Text;

    LiteTable.FilterSQL := edFilterSQL.Text;

    try
      LiteTable.OrderFields:= edOrderFields.Text;
    except
      edOrderFields.SetFocus;
      raise;
    end;
  finally
    ShowState;
  end;

end;

procedure TTableFrame.ShowState;
var
  St: string;

  procedure AddSt(S: string);
  begin
    if St <> '' then
      St:= St + ', ';
    St:= St + S;
  end;

begin
  St:= '';

  if LiteTable.Prepared then begin
    AddSt('Prepared');

    if LiteTable.IsQuery then
      AddSt('IsQuery');
  end;

  if LiteTable.Active then
    AddSt('Active')
  else
    AddSt('Inactive');

  LiteDacForm.StatusBar.Panels[2].Text:= St;
end;

procedure TTableFrame.btOpenClick(Sender: TObject);
begin
  try
    AssignProperties;
    LiteTable.Open;
  finally
    ShowState;
  end;
end;

procedure TTableFrame.btCloseClick(Sender: TObject);
begin
  LiteTable.Close;
  ShowState;
end;

procedure TTableFrame.btPrepareClick(Sender: TObject);
begin
  try
    AssignProperties;
    LiteTable.Prepare;
  finally
    ShowState;
  end;
end;

procedure TTableFrame.btUnPrepareClick(Sender: TObject);
begin
  LiteTable.UnPrepare;
  ShowState;
end;

procedure TTableFrame.btExecuteClick(Sender: TObject);
begin
  try
    AssignProperties;
    LiteTable.Execute;
  finally
    ShowState;
  end;
end;

procedure TTableFrame.edTableNameExit(Sender: TObject);
begin
  AssignProperties;
end;

procedure TTableFrame.edTableNameDropDown(Sender: TObject);
var
  List: TStringList;
begin
  List := TStringList.Create;
  try
    LiteTable.Connection.GetTableNames(List);
    AssignStrings(List, edTableName.Items);
  finally
    List.Free;
  end;
end;

// Demo management
procedure TTableFrame.Initialize;
begin
  inherited;
  LiteTable.Connection := Connection as TLiteConnection;
  edTableName.Text:= LiteTable.TableName;
  edOrderFields.Text:= LiteTable.OrderFields;
  ShowState;
end;

procedure TTableFrame.SetDebug(Value: boolean);
begin
  LiteTable.Debug:= Value;
end;

{$IFDEF FPC}
initialization
  {$i Table.lrs}
{$ENDIF}
end.
