
//////////////////////////////////////////////////
//  SQLite Data Access Components
//  Copyright  2008-2015 Devart. All right reserved.
//////////////////////////////////////////////////

{$IFNDEF CLR}

unit LiteConnectForm;
{$ENDIF}

interface

uses
  Classes, SysUtils,
{$IFDEF MSWINDOWS}
  Windows, Messages,
{$ENDIF}
  CRTypes,
  Graphics, Controls, StdCtrls, ExtCtrls, Forms, Dialogs, TypInfo,
  DBAccess, LiteAccess;

{$IFDEF MSWINDOWS}
const
  WM_SETDATABASETEXT = WM_USER + 1;
{$ENDIF}

type
  TLiteConnectForm = class(TForm)
    Panel: TPanel;
    pPassword: TPanel;
    pClientLibrary: TPanel;
    pDatabase: TPanel;
    edPassword: TEdit;
    lbPassword: TLabel;
    lbClientLibrary: TLabel;
    edClientLibrary: TEdit;
    lbDatabase: TLabel;
    edDatabase: TComboBox;
    pButtons: TPanel;
    btCancel: TButton;
    btConnect: TButton;
    pDirect: TPanel;
    lbDirect: TLabel;
    cbDirect: TCheckBox;

{$IFDEF MSWINDOWS}
    procedure WMSetDataBaseText(var Message: TMessage); message WM_SETDATABASETEXT;
{$ENDIF}

    procedure btConnectClick(Sender: TObject);
    procedure edExit(Sender: TObject);
    procedure edDatabaseChange(Sender: TObject);
    procedure cbDirectClick(Sender: TObject);

  private
    FConnectDialog: TCustomConnectDialog;
    FRetries: integer;
    FRetry: boolean;
    FInInit: boolean;
    FOldDatabaseName: string;
    
    procedure SetConnectDialog(Value: TCustomConnectDialog);

  protected
    procedure DoInit; virtual;
    procedure DoConnect; virtual;

  public
  published
    property ConnectDialog: TCustomConnectDialog read FConnectDialog write SetConnectDialog;
  end;

implementation

{$IFNDEF FPC}
{$IFDEF IDE}
{$R *.dfm}
{$ELSE}
{$R LiteConnectForm.dfm}
{$ENDIF}
{$ELSE}
{$R *.lfm}
{$ENDIF}

uses
  LiteConsts, LiteDacVcl;

{ TLiteConnectForm }

procedure TLiteConnectForm.btConnectClick(Sender: TObject);
begin
  DoConnect;
end;

procedure TLiteConnectForm.cbDirectClick(Sender: TObject);
begin
  lbClientLibrary.Enabled := not cbDirect.Checked;
  edClientLibrary.Enabled := not cbDirect.Checked;
end;

procedure TLiteConnectForm.DoConnect;
begin
  try
    edExit(nil);
    FConnectDialog.Connection.PerformConnect(FRetry);
    ModalResult := mrOk;
  except
    on E: Exception{EDAError} do begin
      Dec(FRetries);
      FRetry := True;
      if FRetries = 0 then
        ModalResult:= mrCancel;
      raise;
    end;
  end;
end;

procedure TLiteConnectForm.DoInit;
var
  CurTop, i: integer;
  ConnectDialogOptions: TConnectDialogOptionArray;
  CurPanel: TPanel;
  CurControl: TWinControl;
  NeedSetActiveControl: Boolean;
begin
  FInInit := True;
  FRetry := False;
  FRetries := FConnectDialog.Retries;
  Caption := FConnectDialog.Caption;

  FConnectDialog.GetOptions(ConnectDialogOptions);
  CurTop := 0;
  NeedSetActiveControl := True;
  for i := Low(ConnectDialogOptions) to High(ConnectDialogOptions) do begin
    CurPanel := nil;
    CurControl := nil;
    case ConnectDialogOptions[i].Kind of
      okPassword: begin
        pPassword.Visible := ConnectDialogOptions[i].Visible;
        CurPanel := pPassword;
        CurControl := edPassword;
        lbPassword.Caption := ConnectDialogOptions[i].Caption;
      end;
      okDatabase: begin
        pDatabase.Visible := ConnectDialogOptions[i].Visible;
        CurPanel := pDatabase;
        CurControl := edDatabase;
        lbDatabase.Caption := ConnectDialogOptions[i].Caption;
      end;
      okDirect: begin
        pDirect.Visible := ConnectDialogOptions[i].Visible;
        CurPanel := pDirect;
        CurControl := cbDirect;
        lbDirect.Caption := ConnectDialogOptions[i].Caption;
      end;
      okClientLibrary: begin
        pClientLibrary.Visible := ConnectDialogOptions[i].Visible;
        CurPanel := pClientLibrary;
        CurControl := edClientLibrary;
        lbClientLibrary.Caption := ConnectDialogOptions[i].Caption;
      end;
    else
      Assert(False);
    end;
    CurPanel.TabOrder := i;
    if CurPanel.Visible then begin
      if i = 0 then
        CurPanel.Top := CurTop + 2
      else
        CurPanel.Top := CurTop;
      CurTop := CurTop + CurPanel.Height;
    end;
    if NeedSetActiveControl and CurPanel.Visible then begin
      ActiveControl := CurControl;
      NeedSetActiveControl := False;
    end;
  end;
  Panel.Height := CurTop + 2;
  pButtons.Top := Panel.Height + 8;
  ClientHeight := pButtons.Top + pButtons.Height + 2;

  btConnect.Caption := FConnectDialog.ConnectButton;
  btCancel.Caption := FConnectDialog.CancelButton;

  edPassword.Text := FConnectDialog.Connection.Password;

  if edDatabase.Visible and (FConnectDialog is TLiteConnectDialog) then begin
  {$IFDEF MSWINDOWS}
    TLiteConnectDialog(FConnectDialog).LoadServerListFromRegistry(edDatabase.Items);
  {$ENDIF}
    edDatabase.Items.Add(SDBBrowse);
  end;

  if (edDatabase.Text <> '') and (edPassword.Text = '') and pPassword.Visible then
    ActiveControl := edPassword;

  if FConnectDialog.Connection is TLiteConnection then begin
    FOldDatabaseName := TLiteConnection(FConnectDialog.Connection).Database;
    edDatabase.Text := FOldDatabaseName;
    edPassword.Text := TLiteConnection(FConnectDialog.Connection).EncryptionKey;
    cbDirect.Checked := TLiteConnection(FConnectDialog.Connection).Options.Direct;
    cbDirectClick(cbDirect);
    edClientLibrary.Text := TLiteConnection(FConnectDialog.Connection).ClientLibrary;
  end;

  FInInit := false;
end;

procedure TLiteConnectForm.edDatabaseChange(Sender: TObject);
var
  DatabaseName: string;
begin
  if FInInit then
    Exit;

  if edDatabase.Text = SDBBrowse then begin
    DatabaseName := '';
    if FConnectDialog is TLiteConnectDialog then
      DatabaseName := TLiteConnectDialog(FConnectDialog).BrowseForDatabase;
    if DatabaseName <> '' then
      FOldDatabaseName := DatabaseName;
{$IFDEF MSWINDOWS}
    PostMessage(Handle, WM_SETDATABASETEXT, 0, 0);
{$ENDIF}
  end;
end;

procedure TLiteConnectForm.edExit(Sender: TObject);
begin
  try
    if FConnectDialog.Connection is TLiteConnection then begin
      if pDirect.Visible then
        TLiteConnection(FConnectDialog.Connection).Options.Direct := cbDirect.Checked;
      if pPassword.Visible then
        TLiteConnection(FConnectDialog.Connection).EncryptionKey := edPassword.Text;
      if pDatabase.Visible then
        TLiteConnection(FConnectDialog.Connection).Database := edDatabase.Text;
      if pClientLibrary.Visible then
        TLiteConnection(FConnectDialog.Connection).ClientLibrary := edClientLibrary.Text;
    end;
  except
    ActiveControl := Sender as TWinControl;
    raise;
  end;
end;

procedure TLiteConnectForm.SetConnectDialog(Value: TCustomConnectDialog);
begin
  FConnectDialog := Value;
  DoInit;
end;

{$IFDEF MSWINDOWS}
procedure TLiteConnectForm.WMSetDataBaseText(var Message: TMessage);
begin
  edDatabase.SetFocus;
  edDatabase.Text := FOldDatabaseName;
  edDatabase.SelectAll;
end;
{$ENDIF}

initialization
  if GetClass('TLiteConnectForm') = nil then
    Classes.RegisterClass(TLiteConnectForm);

end.
